@@ -1,8 +1,5 @@
-#ifdef _OPENMP
-#include <omp.h>
-#endif
+#include "collapse_c.h" // Needs to be first because includes OpenMP
 #include "kit.h"
-#include "collapse_c.h"
 
 // C-implementations for different data types ----------------------------------
 // TODO: outsource and memset hash table?
@@ -244,7 +241,7 @@ int ndistinct_impl_int(SEXP x, int narm) {
     case INTSXP:  // TODO: optimize for plain integer??
       return isFactor(x) ? ndistinct_fct(INTEGER(x), &l, l, nlevels(x), 1, narm) :
                            ndistinct_int(INTEGER(x), &l, l, 1, narm);
-    case LGLSXP: return ndistinct_logi(INTEGER(x), &l, l, 1, narm);
+    case LGLSXP: return ndistinct_logi(LOGICAL(x), &l, l, 1, narm);
     case STRSXP: return ndistinct_string(STRING_PTR(x), &l, l, 1, narm);
     default: error("Not Supported SEXP Type: '%s'", type2char(TYPEOF(x)));
   }
@@ -352,7 +349,7 @@ SEXP ndistinct_g_impl(SEXP x, const int ng, const int *restrict pgs, const int *
 SEXP fndistinctC(SEXP x, SEXP g, SEXP Rnarm, SEXP Rnthreads) {
   if(isNull(g)) return ndistinct_impl(x, asLogical(Rnarm));
   if(TYPEOF(g) != VECSXP || !inherits(g, "GRP")) error("g needs to be an object of class 'GRP', see ?GRP");
-  const SEXP *restrict pg = SEXPPTR(g), o = pg[6];
+  const SEXP *restrict pg = SEXPPTR_RO(g), o = pg[6];
   SEXP res;
   int sorted = LOGICAL(pg[5])[1] == 1, ng = INTEGER(pg[0])[0], *restrict pgs = INTEGER(pg[2]), *restrict po, *restrict pst,
     l = length(x), nthreads = asInteger(Rnthreads);
@@ -388,7 +385,8 @@ SEXP fndistinctlC(SEXP x, SEXP g, SEXP Rnarm, SEXP Rdrop, SEXP Rnthreads) {
   if(l < 1) return ScalarInteger(0);
   if(nthreads > max_threads) nthreads = max_threads;
   if(isNull(g) && asLogical(Rdrop)) {
-    SEXP out = PROTECT(allocVector(INTSXP, l)), *restrict px = SEXPPTR(x);
+    SEXP out = PROTECT(allocVector(INTSXP, l));
+    const SEXP *restrict px = SEXPPTR_RO(x);
     int *restrict pout = INTEGER(out);
     if(nthreads <= 1) {
       for(int j = 0; j != l; ++j) pout[j] = ndistinct_impl_int(px[j], narm);
@@ -401,8 +399,8 @@ SEXP fndistinctlC(SEXP x, SEXP g, SEXP Rnarm, SEXP Rdrop, SEXP Rnthreads) {
     UNPROTECT(1);
     return out;
   } else {
-    SEXP out = PROTECT(allocVector(VECSXP, l)), sym_label = PROTECT(install("label")),
-      *restrict pout = SEXPPTR(out), *restrict px = SEXPPTR(x);
+    SEXP out = PROTECT(allocVector(VECSXP, l)), sym_label = PROTECT(install("label")), *restrict pout = SEXPPTR(out);
+    const SEXP *restrict px = SEXPPTR_RO(x);
     if(isNull(g)) {
       if(nthreads <= 1) {
         for(int j = 0; j != l; ++j) pout[j] = ndistinct_impl(px[j], narm);
@@ -420,7 +418,7 @@ SEXP fndistinctlC(SEXP x, SEXP g, SEXP Rnarm, SEXP Rdrop, SEXP Rnthreads) {
       DFcopyAttr(out, x, /*ng=*/0);
     } else {
       if(TYPEOF(g) != VECSXP || !inherits(g, "GRP")) error("g needs to be an object of class 'GRP', see ?GRP");
-      const SEXP *restrict pg = SEXPPTR(g), o = pg[6];
+      const SEXP *restrict pg = SEXPPTR_RO(g), o = pg[6];
       int sorted = LOGICAL(pg[5])[1] == 1, ng = INTEGER(pg[0])[0], *restrict pgs = INTEGER(pg[2]), *restrict po, *restrict pst, gl = length(pg[1]);
       if(isNull(o)) {
         int *cgs = (int *) R_alloc(ng+2, sizeof(int)), *restrict pgv = INTEGER(pg[1]); cgs[1] = 1;
@@ -499,7 +497,7 @@ SEXP fndistinctmC(SEXP x, SEXP g, SEXP Rnarm, SEXP Rdrop, SEXP Rnthreads) {
     return res;
   } else { // With groups
     if(TYPEOF(g) != VECSXP || !inherits(g, "GRP")) error("g needs to be an object of class 'GRP', see ?GRP");
-    const SEXP *restrict pg = SEXPPTR(g), o = pg[6];
+    const SEXP *restrict pg = SEXPPTR_RO(g), o = pg[6];
     int sorted = LOGICAL(pg[5])[1] == 1, ng = INTEGER(pg[0])[0], *restrict pgs = INTEGER(pg[2]), *restrict po, *restrict pst, gl = length(pg[1]);
     if(l != gl) error("length(g) must match nrow(x)");
 
