@@ -1,30 +1,25 @@
-#ifdef _OPENMP
-#include <omp.h>
-#endif
 #include "collapse_c.h"
 // #include <R_ext/Altrep.h>
 
 double fsum_double_impl(const double *restrict px, const int narm, const int l) {
   double sum;
   if(narm == 1) {
-    // Somehow this is faster than forward loop...
-    int j = l-1;
-    sum = px[j];
-    while(ISNAN(sum) && j!=0) sum = px[--j];
-    if(j != 0) for(int i = j; i--; ) {
-      if(ISNAN(px[i])) continue;
-      sum += px[i];
+    int j = 1;
+    sum = px[0];
+    while(ISNAN(sum) && j!=l) sum = px[j++];
+    if(j != l) {
+      #pragma omp simd reduction(+:sum)
+      for(int i = j; i < l; ++i) sum += NISNAN(px[i]) ? px[i] : 0.0;
     }
   } else {
     sum = 0;
     if(narm) {
-      for(int i = l; i--; ) {
-        if(ISNAN(px[i])) continue; // Somehow continue statements with backwards loops are faster...
-        sum += px[i];
-      }
+      #pragma omp simd reduction(+:sum)
+      for(int i = 0; i < l; ++i) sum += NISNAN(px[i]) ? px[i] : 0.0;
     } else {
      // Should just be fast, don't stop for NA's
-      for(int i = 0; i != l; ++i) sum += px[i];
+      #pragma omp simd reduction(+:sum)
+      for(int i = 0; i < l; ++i) sum += px[i];
     }
   }
   return sum;
@@ -57,12 +52,12 @@ double fsum_double_omp_impl(const double *restrict px, const int narm, const int
     sum = px[0];
     while(ISNAN(sum) && j != l) sum = px[j++];
     if(j != l) {
-      #pragma omp parallel for num_threads(nthreads) reduction(+:sum)
-      for(int i = j; i < l; ++i) if(NISNAN(px[i])) sum += px[i]; // Fastest ?
-    } else if(narm == 2) sum = 0;
+      #pragma omp parallel for simd num_threads(nthreads) reduction(+:sum)
+      for(int i = j; i < l; ++i) sum += NISNAN(px[i]) ? px[i] : 0.0;
+    } else if(narm == 2) sum = 0.0;
   } else {
     sum = 0;
-    #pragma omp parallel for num_threads(nthreads) reduction(+:sum)
+    #pragma omp parallel for simd num_threads(nthreads) reduction(+:sum)
     for(int i = 0; i < l; ++i) sum += px[i]; // Cannot have break statements in OpenMP for loop
   }
   return sum;
@@ -92,19 +87,22 @@ double fsum_double_omp_impl(const double *restrict px, const int narm, const int
 double fsum_weights_impl(const double *restrict px, const double *restrict pw, const int narm, const int l) {
   double sum;
   if(narm == 1) {
-    int j = l-1;
-    while((ISNAN(px[j]) || ISNAN(pw[j])) && j!=0) --j;
+    int j = 0, end = l-1;
+    while((ISNAN(px[j]) || ISNAN(pw[j])) && j!=end) ++j;
     sum = px[j] * pw[j];
-    if(j != 0) {
-      for(int i = j; i--; ) if(NISNAN(px[i]) && NISNAN(pw[i])) sum += px[i] * pw[i];
+    if(j != end) {
+      #pragma omp simd reduction(+:sum)
+      for(int i = j+1; i < l; ++i) sum += (NISNAN(px[i]) && NISNAN(pw[i])) ? px[i] * pw[i] : 0.0;
     }
   } else {
     sum = 0;
     if(narm) {
-      for(int i = 0; i != l; ++i) if(NISNAN(px[i]) && NISNAN(pw[i])) sum += px[i] * pw[i];
+      #pragma omp simd reduction(+:sum)
+      for(int i = 0; i < l; ++i) sum += (NISNAN(px[i]) && NISNAN(pw[i])) ? px[i] * pw[i] : 0.0;
     } else {
       // Also here speed is key...
-      for(int i = 0; i != l; ++i) sum += px[i] * pw[i];
+      #pragma omp simd reduction(+:sum)
+      for(int i = 0; i < l; ++i) sum += px[i] * pw[i];
     }
   }
   return sum;
@@ -134,17 +132,15 @@ double fsum_weights_omp_impl(const double *restrict px, const double *restrict p
   double sum;
   if(narm) {
     int j = 0;
-    while((ISNAN(px[j]) || ISNAN(pw[j])) && j!=l) ++j;
+    while(j!=l && (ISNAN(px[j]) || ISNAN(pw[j]))) ++j;
     if(j != l) {
       sum = px[j] * pw[j];
-      #pragma omp parallel for num_threads(nthreads) reduction(+:sum)
-      for(int i = j+1; i < l; ++i) {
-        if(NISNAN(px[i]) && NISNAN(pw[i])) sum += px[i] * pw[i];
-      }
-    } else sum = narm == 1 ? NA_REAL : 0;
+      #pragma omp parallel for simd num_threads(nthreads) reduction(+:sum)
+      for(int i = j+1; i < l; ++i) sum += (NISNAN(px[i]) && NISNAN(pw[i])) ? px[i] * pw[i] : 0.0;
+    } else sum = narm == 1 ? NA_REAL : 0.0;
   } else {
     sum = 0;
-    #pragma omp parallel for num_threads(nthreads) reduction(+:sum)
+    #pragma omp parallel for simd num_threads(nthreads) reduction(+:sum)
     for(int i = 0; i < l; ++i) sum += px[i] * pw[i];
   }
   return sum;
@@ -238,12 +234,12 @@ double fsum_int_omp_impl(const int *restrict px, const int narm, const int l, co
     while(px[j] == NA_INTEGER && j!=l) ++j;
     if(j == l && (l > 1 || px[j-1] == NA_INTEGER)) return narm == 1 ? NA_REAL : 0;
     sum = (long long)px[j];
-    #pragma omp parallel for num_threads(nthreads) reduction(+:sum)
-    for(int i = j+1; i < l; ++i) if(px[i] != NA_INTEGER) sum += (long long)px[i];
+    #pragma omp parallel for simd num_threads(nthreads) reduction(+:sum)
+    for(int i = j+1; i < l; ++i) sum += px[i] != NA_INTEGER ? (long long)px[i] : 0;
   } else {
     sum = 0;
-    #pragma omp parallel for num_threads(nthreads) reduction(+:sum)
-    for(int i = 0; i < l; ++i) if(px[i] != NA_INTEGER) sum += (long long)px[i]; // Need this, else wrong result
+    #pragma omp parallel for simd num_threads(nthreads) reduction(+:sum)
+    for(int i = 0; i < l; ++i) sum += px[i] != NA_INTEGER ? (long long)px[i] : 0; // Need this, else wrong result
     if(sum == 0 && px[0] == NA_INTEGER) return NA_REAL; // Problem here, could be NA_INTEGER mixed with 0's, but very rare...
   }
   return (double)sum;
@@ -637,7 +633,8 @@ SEXP fsumlC(SEXP x, SEXP Rng, SEXP g, SEXP w, SEXP Rnarm, SEXP fill, SEXP Rdrop,
   }
 
   if(ng == 0 && asLogical(Rdrop)) {
-    SEXP out = PROTECT(allocVector(REALSXP, l)), *restrict px = SEXPPTR(x);
+    SEXP out = PROTECT(allocVector(REALSXP, l));
+    const SEXP *restrict px = SEXPPTR_RO(x);
     double *restrict pout = REAL(out);
     COLWISE_FSUM_LIST(fsum_impl_dbl, fsum_w_impl_dbl);
     setAttrib(out, R_NamesSymbol, getAttrib(x, R_NamesSymbol));
@@ -645,7 +642,8 @@ SEXP fsumlC(SEXP x, SEXP Rng, SEXP g, SEXP w, SEXP Rnarm, SEXP fill, SEXP Rdrop,
     return out;
   }
 
-  SEXP out = PROTECT(allocVector(VECSXP, l)), *restrict pout = SEXPPTR(out), *restrict px = SEXPPTR(x);
+  SEXP out = PROTECT(allocVector(VECSXP, l)), *restrict pout = SEXPPTR(out);
+  const SEXP *restrict px = SEXPPTR_RO(x);
 
   if(ng == 0) {
     COLWISE_FSUM_LIST(fsum_impl_SEXP, fsum_w_impl_SEXP);
@@ -682,7 +680,7 @@ SEXP fsumlC(SEXP x, SEXP Rng, SEXP g, SEXP w, SEXP Rnarm, SEXP fill, SEXP Rdrop,
           if(ATTRIB(xj) != R_NilValue && !(isObject(xj) && inherits(xj, "ts"))) copyMostAttrib(xj, outj);
           if(TYPEOF(xj) != REALSXP) {
             if(TYPEOF(xj) != INTSXP && TYPEOF(xj) != LGLSXP) error("Unsupported SEXP type: '%s'", type2char(TYPEOF(xj)));
-            if(dup == 0) {x = PROTECT(shallow_duplicate(x)); ++nprotect; px = SEXPPTR(x); dup = 1;}
+            if(dup == 0) {x = PROTECT(shallow_duplicate(x)); ++nprotect; px = SEXPPTR_RO(x); dup = 1;}
             SET_VECTOR_ELT(x, j, coerceVector(xj, REALSXP));
           }
         }
